@@ -36,6 +36,7 @@ static FILE * playlist_wav = NULL;
 static FILE * playlist_mp3 = NULL;
 static FILE * playlist_ogg = NULL;
 static FILE * playlist_flac = NULL;
+static FILE * playlist_wavpack = NULL;
 
 /* ripping or encoding, so that can know not to clear the tracklist on eject */
 bool working;
@@ -53,6 +54,7 @@ static double rip_percent;
 static double mp3_percent;
 static double ogg_percent;
 static double flac_percent;
+static double wavpack_percent;
 static int rip_tracks_completed;
 static int encode_tracks_completed;
 
@@ -74,31 +76,23 @@ void abort_threads()
         kill(flac_pid, SIGKILL);
     
     /* wait until all the worker threads are done */
-    while (cdparanoia_pid != 0 || lame_pid != 0 || oggenc_pid != 0 || flac_pid != 0)
+    while (cdparanoia_pid != 0 || lame_pid != 0 || oggenc_pid != 0 || flac_pid != 0 || wavpack_pid != 0)
     {
-#ifdef DEBUG
-        printf("w1");
-#endif
+        debugLog("w1");
         usleep(100000);
     }
     
     g_cond_signal(available);
     
-#ifdef DEBUG
-    printf("Aborting: 1\n");
-#endif
+    debugLog("Aborting: 1\n");
     g_thread_join(ripper);
-#ifdef DEBUG
-    printf("Aborting: 2\n");
-#endif
+    debugLog("Aborting: 2\n");
     g_thread_join(encoder);
-#ifdef DEBUG
-    printf("Aborting: 3\n");
-#endif
+    debugLog("Aborting: 3\n");
     g_thread_join(tracker);
-#ifdef DEBUG
-    printf("Aborting: 4 (All threads joined)\n");
-#endif
+    debugLog("Aborting: 4 (All threads joined)\n");
+    
+    gtk_window_set_title(GTK_WINDOW(win_main), "Asunder");
     
     gtk_widget_hide(win_ripping);
     gdk_flush();
@@ -120,6 +114,7 @@ void dorip()
     mp3_percent = 0.0;
     ogg_percent = 0.0;
     flac_percent = 0.0;
+    wavpack_percent = 0.0;
     rip_tracks_completed = 0;
     encode_tracks_completed = 0;
     
@@ -132,7 +127,8 @@ void dorip()
     overwriteNone = false;
     
     // make sure there's at least one format to rip to
-    if (!global_prefs->rip_wav && !global_prefs->rip_mp3 && !global_prefs->rip_ogg && !global_prefs->rip_flac)
+    if (!global_prefs->rip_wav && !global_prefs->rip_mp3 && !global_prefs->rip_ogg && 
+        !global_prefs->rip_flac && !global_prefs->rip_wavpack)
     {
         GtkWidget * dialog;
         dialog = gtk_message_dialog_new(GTK_WINDOW(win_main), 
@@ -181,9 +177,7 @@ void dorip()
     
     /* CREATE the album directory */
     char * dirpath = make_filename(prefs_get_music_dir(global_prefs), albumdir, NULL, NULL);
-#ifdef DEBUG
-    printf("Making album directory '%s'\n", dirpath);
-#endif
+    debugLog("Making album directory '%s'\n", dirpath);
     
     if ( recursive_mkdir(dirpath, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH) != 0 && 
          errno != EEXIST )
@@ -208,178 +202,50 @@ void dorip()
     
     if (global_prefs->make_playlist)
     {
-        bool makePlaylist;
-        int rc;
-        struct stat statStruct;
-#ifdef DEBUG
-        printf("Creating playlists\n");
-#endif
+        debugLog("Creating playlists\n");
+        
         if (global_prefs->rip_wav)
         {
             char * filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, playlist, "wav.m3u");
-            // don't need the following because playlists are only allowed in the album dir
-            //recursive_parent_mkdir(filename, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
             
-            rc = stat(filename, &statStruct);
-            if(rc == 0)
-            {
-                if(confirmOverwrite(filename))
-                    makePlaylist = true;
-                else
-                    makePlaylist = false;
-            }
-            else
-                makePlaylist = true;
+            make_playlist(filename, &playlist_wav);
             
-            if(makePlaylist)
-            {
-                playlist_wav = fopen(filename, "w");
-                
-                if (playlist_wav == NULL)
-                {
-                    GtkWidget * dialog;
-                    dialog = gtk_message_dialog_new(GTK_WINDOW(win_main), 
-                                                    GTK_DIALOG_DESTROY_WITH_PARENT, 
-                                                    GTK_MESSAGE_ERROR, 
-                                                    GTK_BUTTONS_OK, 
-                                                    "Unable to create WAV playlist \"%s\": %s", 
-                                                    filename, strerror(errno));
-                    gtk_dialog_run(GTK_DIALOG(dialog));
-                    gtk_widget_destroy(dialog);
-                } else {
-                    fprintf(playlist_wav, "#EXTM3U\n");
-                }
-            }
-            else
-                playlist_wav = NULL;
-
             free(filename);
         }
         if (global_prefs->rip_mp3)
         {
             char * filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, playlist, "mp3.m3u");
-            // don't need the following because playlists are only allowed in the album dir
-            //recursive_parent_mkdir(filename, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
             
-            rc = stat(filename, &statStruct);
-            if(rc == 0)
-            {
-                if(confirmOverwrite(filename))
-                    makePlaylist = true;
-                else
-                    makePlaylist = false;
-            }
-            else
-                makePlaylist = true;
+            make_playlist(filename, &playlist_mp3);
             
-            if(makePlaylist)
-            {
-                playlist_mp3 = fopen(filename, "w");
-                
-                if (playlist_mp3 == NULL)
-                {
-                    GtkWidget * dialog;
-                    dialog = gtk_message_dialog_new(GTK_WINDOW(win_main), 
-                                                    GTK_DIALOG_DESTROY_WITH_PARENT, 
-                                                    GTK_MESSAGE_ERROR, 
-                                                    GTK_BUTTONS_OK, 
-                                                    "Unable to create MP3 playlist \"%s\": %s", 
-                                                    filename, strerror(errno));
-                    gtk_dialog_run(GTK_DIALOG(dialog));
-                    gtk_widget_destroy(dialog);
-                } else {
-                    fprintf(playlist_mp3, "#EXTM3U\n");
-                }
-            }
-            else
-                playlist_mp3 = NULL;
-
             free(filename);
         }
         if (global_prefs->rip_ogg)
         {
             char * filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, playlist, "ogg.m3u");
-            // don't need the following because playlists are only allowed in the album dir
-            //recursive_parent_mkdir(filename, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
             
-            rc = stat(filename, &statStruct);
-            if(rc == 0)
-            {
-                if(confirmOverwrite(filename))
-                    makePlaylist = true;
-                else
-                    makePlaylist = false;
-            }
-            else
-                makePlaylist = true;
+            make_playlist(filename, &playlist_ogg);
             
-            if(makePlaylist)
-            {
-                playlist_ogg = fopen(filename, "w");
-                
-                if (playlist_ogg == NULL)
-                {
-                    GtkWidget * dialog;
-                    dialog = gtk_message_dialog_new(GTK_WINDOW(win_main), 
-                                                    GTK_DIALOG_DESTROY_WITH_PARENT, 
-                                                    GTK_MESSAGE_ERROR, 
-                                                    GTK_BUTTONS_OK, 
-                                                    "Unable to create OGG playlist \"%s\": %s", 
-                                                    filename, strerror(errno));
-                    gtk_dialog_run(GTK_DIALOG(dialog));
-                    gtk_widget_destroy(dialog);
-                } else {
-                    fprintf(playlist_ogg, "#EXTM3U\n");
-                }
-            }
-            else
-                playlist_ogg = NULL;
-
             free(filename);
         }
         if (global_prefs->rip_flac)
         {
             char * filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, playlist, "flac.m3u");
-            // don't need the following because playlists are only allowed in the album dir
-            //recursive_parent_mkdir(filename, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
             
-            rc = stat(filename, &statStruct);
-            if(rc == 0)
-            {
-                if(confirmOverwrite(filename))
-                    makePlaylist = true;
-                else
-                    makePlaylist = false;
-            }
-            else
-                makePlaylist = true;
+            make_playlist(filename, &playlist_flac);
+            
+            free(filename);
+        }
+        if (global_prefs->rip_wavpack)
+        {
+            char * filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, playlist, "wv.m3u");
+            
+            make_playlist(filename, &playlist_wavpack);
             
-            if(makePlaylist)
-            {
-                playlist_flac = fopen(filename, "w");
-                
-                if (playlist_flac == NULL)
-                {
-                    GtkWidget * dialog;
-                    dialog = gtk_message_dialog_new(GTK_WINDOW(win_main), 
-                                                    GTK_DIALOG_DESTROY_WITH_PARENT, 
-                                                    GTK_MESSAGE_ERROR, 
-                                                    GTK_BUTTONS_OK, 
-                                                    "Unable to create FLAC playlist \"%s\": %s", 
-                                                    filename, strerror(errno));
-                    gtk_dialog_run(GTK_DIALOG(dialog));
-                    gtk_widget_destroy(dialog);
-                } else {
-                    fprintf(playlist_flac, "#EXTM3U\n");
-                }
-            }
-            else
-                playlist_flac = NULL;
-
             free(filename);
         }
     }
-
+    
     free(albumdir);
     free(playlist);
     
@@ -389,11 +255,13 @@ void dorip()
     numLameFailed = 0;
     numOggFailed = 0;
     numFlacFailed = 0;
+    numWavpackFailed = 0;
     
     numCdparanoiaOk = 0;
     numLameOk = 0;
     numOggOk = 0;
     numFlacOk = 0;
+    numWavpackOk = 0;
     
     ripper = g_thread_create(rip, NULL, TRUE, NULL);
     encoder = g_thread_create(encode, NULL, TRUE, NULL);
@@ -444,9 +312,8 @@ gpointer rip(gpointer data)
             musicfilename = parse_format(global_prefs->format_music, tracknum, trackartist, albumtitle, tracktitle);
             wavfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wav");
             
-#ifdef DEBUG
-            printf("Ripping track %d to \"%s\"\n", tracknum, wavfilename);
-#endif
+            debugLog("Ripping track %d to \"%s\"\n", tracknum, wavfilename);
+            
             if (aborted) g_thread_exit(NULL);
             
             struct stat statStruct;
@@ -527,6 +394,8 @@ gpointer encode(gpointer data)
     char * mp3filename = NULL;
     char * oggfilename = NULL;
     char * flacfilename = NULL;
+    char * wavpackfilename = NULL;
+    char * wavpackfilename2 = NULL;
     struct stat statStruct;
     bool doEncode;
     
@@ -591,12 +460,13 @@ gpointer encode(gpointer data)
             mp3filename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "mp3");
             oggfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "ogg");
             flacfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "flac");
-        
+            wavpackfilename = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wv");
+            wavpackfilename2 = make_filename(prefs_get_music_dir(global_prefs), albumdir, musicfilename, "wvc");
+            
             if (global_prefs->rip_mp3)
             {
-#ifdef DEBUG
-                printf("Encoding track %d to \"%s\"\n", tracknum, mp3filename);
-#endif
+                debugLog("Encoding track %d to \"%s\"\n", tracknum, mp3filename);
+                
                 if (aborted) g_thread_exit(NULL);
                 
                 rc = stat(mp3filename, &statStruct);
@@ -627,9 +497,8 @@ gpointer encode(gpointer data)
             }
             if (global_prefs->rip_ogg)
             {
-#ifdef DEBUG
-                printf("Encoding track %d to \"%s\"\n", tracknum, oggfilename);
-#endif
+                debugLog("Encoding track %d to \"%s\"\n", tracknum, oggfilename);
+                
                 if (aborted) g_thread_exit(NULL);
                 
                 rc = stat(oggfilename, &statStruct);
@@ -660,9 +529,8 @@ gpointer encode(gpointer data)
             }
             if (global_prefs->rip_flac)
             {
-#ifdef DEBUG
-                printf("Encoding track %d to \"%s\"\n", tracknum, flacfilename);
-#endif
+                debugLog("Encoding track %d to \"%s\"\n", tracknum, flacfilename);
+                
                 if (aborted) g_thread_exit(NULL);
                 
                 rc = stat(flacfilename, &statStruct);
@@ -692,11 +560,47 @@ gpointer encode(gpointer data)
                     fflush(playlist_flac);
                 }
             }
+            if (global_prefs->rip_wavpack)
+            {
+                debugLog("Encoding track %d to \"%s\"\n", tracknum, wavpackfilename);
+                
+                if (aborted) g_thread_exit(NULL);
+                
+                rc = stat(wavpackfilename, &statStruct);
+                int rc2 = stat(wavpackfilename2, &statStruct);
+                if(rc == 0 || rc2 == 0)
+                {
+                    gdk_threads_enter();
+                        if(confirmOverwrite(wavpackfilename))
+                            doEncode = true;
+                        else
+                            doEncode = false;
+                    gdk_threads_leave();
+                }
+                else
+                    doEncode = true;
+                
+                if(doEncode)
+                {
+                    wavpack(tracknum, wavfilename, 
+                            global_prefs->wavpack_compression, global_prefs->wavpack_hybrid, 
+                            int_to_wavpack_bitrate(global_prefs->wavpack_bitrate), &wavpack_percent);
+                }
+                
+                if (aborted) g_thread_exit(NULL);
+                
+                if (playlist_wavpack)
+                {
+                    for (i=strlen(flacfilename); ((i>0) && (wavpackfilename[i] != '/')); i--);
+                    fprintf(playlist_wavpack, "#EXTINF:%d,%s - %s\n", (min*60)+sec, trackartist, tracktitle);
+                    fprintf(playlist_wavpack, "%s\n", basename(wavpackfilename));
+                    fflush(playlist_wavpack);
+                }
+            }
             if (!global_prefs->rip_wav)
             {
-#ifdef DEBUG
-                printf("Removing track %d WAV file\n", tracknum);
-#endif
+                debugLog("Removing track %d WAV file\n", tracknum);
+                
                 if (unlink(wavfilename) != 0)
                 {
                     printf("Unable to delete WAV file \"%s\": %s\n", wavfilename, strerror(errno));
@@ -717,10 +621,12 @@ gpointer encode(gpointer data)
             free(mp3filename);
             free(oggfilename);
             free(flacfilename);
+            free(wavpackfilename);
             
             mp3_percent = 0.0;
             ogg_percent = 0.0;
             flac_percent = 0.0;
+            wavpack_percent = 0.0;
             encode_tracks_completed++;
         }
         
@@ -749,11 +655,9 @@ gpointer encode(gpointer data)
     available = NULL;
     
     /* wait until all the worker threads are done */
-    while (cdparanoia_pid != 0 || lame_pid != 0 || oggenc_pid != 0 || flac_pid != 0)
+    while (cdparanoia_pid != 0 || lame_pid != 0 || oggenc_pid != 0 || flac_pid != 0 || wavpack_pid != 0)
     {
-#ifdef DEBUG
-        printf("w2");
-#endif
+        debugLog("w2");
         usleep(100000);
     }
     
@@ -763,8 +667,8 @@ gpointer encode(gpointer data)
     gdk_threads_enter();
         gtk_widget_hide(win_ripping);
         gdk_flush();
-        show_completed_dialog(numCdparanoiaOk + numLameOk + numOggOk + numFlacOk,
-                              numCdparanoiaFailed + numLameFailed + numOggFailed + numFlacFailed);
+        show_completed_dialog(numCdparanoiaOk + numLameOk + numOggOk + numFlacOk + numWavpackOk,
+                              numCdparanoiaFailed + numLameFailed + numOggFailed + numFlacFailed + numWavpackFailed);
     gdk_threads_leave();
     
     return NULL;
@@ -780,6 +684,8 @@ gpointer track(gpointer data)
         parts++;
     if(global_prefs->rip_flac) 
         parts++;
+    if(global_prefs->rip_wavpack) 
+        parts++;
     
     gdk_threads_enter();
         GtkProgressBar * progress_total = GTK_PROGRESS_BAR(lookup_widget(win_ripping, "progress_total"));
@@ -802,22 +708,26 @@ gpointer track(gpointer data)
     
     double prip;
     char srip[13];
-    double pencode;
+    double pencode = 0;
     char sencode[13];
     double ptotal;
     char stotal[5];
     char windowTitle[15]; /* "Asunder - 100%" */
-
-    while (!aborted && !allDone)
+    
+    while (!allDone)
     {
-#ifdef DEBUG
-        printf("completed tracks %d, rip %.2f%%; encoded tracks %d, mp3 %.2f%% ogg %.2f%% flac %.2f%%\n", rip_tracks_completed, rip_percent, encode_tracks_completed, mp3_percent, ogg_percent, flac_percent);
-#endif
+        if (aborted) g_thread_exit(NULL);
+        
+        debugLog("completed tracks %d, rip %.2lf%%; encoded tracks %d, "
+                 "mp3 %.2lf%% ogg %.2lf%% flac %.2lf%% wavpack %.2lf%%\n", 
+                 rip_tracks_completed, rip_percent, encode_tracks_completed, 
+                 mp3_percent, ogg_percent, flac_percent, wavpack_percent);
+        
         prip = (rip_tracks_completed+rip_percent) / tracks_to_rip;
         snprintf(srip, 13, "%d%% (%d/%d)", (int)(prip*100), rip_tracks_completed, tracks_to_rip);
         if (parts > 1)
         {
-            pencode = ((double)encode_tracks_completed/(double)tracks_to_rip) + ((mp3_percent+ogg_percent+flac_percent)/(parts-1)/tracks_to_rip);
+            pencode = ((double)encode_tracks_completed/(double)tracks_to_rip) + ((mp3_percent+ogg_percent+flac_percent+wavpack_percent)/(parts-1)/tracks_to_rip);
             snprintf(sencode, 13, "%d%% (%d/%d)", (int)(pencode*100), encode_tracks_completed, tracks_to_rip);
             ptotal = prip/parts + pencode*(parts-1)/parts;
         } else {
@@ -828,6 +738,8 @@ gpointer track(gpointer data)
         strcpy(windowTitle, "Asunder - ");
         strcat(windowTitle, stotal);
         
+        if (aborted) g_thread_exit(NULL);
+        
         gdk_threads_enter();
             gtk_progress_bar_set_fraction(progress_rip, prip);
             gtk_progress_bar_set_text(progress_rip, srip);
@@ -846,8 +758,9 @@ gpointer track(gpointer data)
         usleep(100000);
     }
     
-    gtk_window_set_title(GTK_WINDOW(win_main), "Asunder");
+    gdk_threads_enter();
+        gtk_window_set_title(GTK_WINDOW(win_main), "Asunder");
+    gdk_threads_leave();
     
     return NULL;
 }
-
