@@ -31,16 +31,19 @@ pid_t cdparanoia_pid;
 pid_t lame_pid;
 pid_t oggenc_pid;
 pid_t flac_pid;
+pid_t wavpack_pid;
 
 int numCdparanoiaFailed;
 int numLameFailed;
 int numOggFailed;
 int numFlacFailed;
+int numWavpackFailed;
 
 int numCdparanoiaOk;
 int numLameOk;
 int numOggOk;
 int numFlacOk;
+int numWavpackOk;
 
 int numchildren = 0;
 static bool waitBeforeSigchld;
@@ -81,26 +84,25 @@ void sigchld(int signum)
     /* this is because i can't seem to be able to block sigchld: */
     while(waitBeforeSigchld)
     {
-#ifdef DEBUG
-        printf("waiting in sigchld()\n");
-#endif
+        debugLog("waiting in sigchld()\n");
         usleep(100);
     }
     
-    if (pid != cdparanoia_pid && pid != lame_pid && pid != oggenc_pid && pid != flac_pid)
+    if (pid != cdparanoia_pid && pid != lame_pid && pid != oggenc_pid && 
+        pid != flac_pid && pid != wavpack_pid)
+    {
         printf("SIGCHLD for unknown pid, report bug please\n");
-#ifdef DEBUG
-    printf("%d exited with %d: ", pid, status);
+    }
+    
+    debugLog("%d exited with %d: ", pid, status);
     if (WIFEXITED(status))
-        printf("exited, status=%d\n", WEXITSTATUS(status));
+        debugLog("exited, status=%d\n", WEXITSTATUS(status));
     else if (WIFSIGNALED(status))
-        printf("killed by signal %d\n", WTERMSIG(status));
+        debugLog("killed by signal %d\n", WTERMSIG(status));
     else if (WIFSTOPPED(status))
-        printf("stopped by signal %d\n", WSTOPSIG(status));
+        debugLog("stopped by signal %d\n", WSTOPSIG(status));
     else if (WIFCONTINUED(status))
-        printf("continued\n");
-    fflush(NULL);
-#endif
+        debugLog("continued\n");
 
     if (status != 0)
     {
@@ -125,6 +127,11 @@ void sigchld(int signum)
             flac_pid = 0;
             numFlacFailed++;
         }
+        else if (pid == wavpack_pid)
+        {
+            wavpack_pid = 0;
+            numWavpackFailed++;
+        }
     }
     else
     {
@@ -149,6 +156,11 @@ void sigchld(int signum)
             flac_pid = 0;
             numFlacOk++;
         }
+        else if (pid == wavpack_pid)
+        {
+            wavpack_pid = 0;
+            numWavpackOk++;
+        }
     }
 }
 
@@ -202,16 +214,11 @@ int exec_with_output(const char * args[], int toread, pid_t * p)
         fatalError("exec_with_output(): execvp() failed");
     }
     
-#ifdef DEBUG
     int count;
-    printf("%d started: %s ", *p, args[0]);
+    debugLog("%d started: %s ", *p, args[0]);
     for (count = 1; args[count] != NULL; count++)
-    {
-        printf("%s ", args[count]);
-    }
-    printf("\n");
-    fflush(NULL);
-#endif
+        debugLog("%s ", args[count]);
+    debugLog("\n");
     
     // i'm the parent, get ready to wait for children
     numchildren++;
@@ -269,7 +276,7 @@ void cdparanoia(char * cdrom, int tracknum, char * filename, double * progress)
                 size = 1;
             }
             
-        } while ((buf[pos] != '\n') && (size > 0));
+        } while ((buf[pos] != '\n') && (size > 0) && (pos < 256));
         buf[pos] = '\0';
 
         if ((buf[0] == 'R') && (buf[1] == 'i'))
@@ -291,9 +298,7 @@ void cdparanoia(char * cdrom, int tracknum, char * filename, double * progress)
     /* don't go on until the signal for the previous call is handled */
     while (cdparanoia_pid != 0)
     {
-#ifdef DEBUG
-        printf("w3");
-#endif
+        debugLog("w3\n");
         usleep(100000);
     }
     
@@ -368,7 +373,8 @@ void lame(int tracknum,
         args[pos++] = "--tt";
         args[pos++] = title;
     }
-    if ((genre != NULL) && (strlen(genre) > 0))
+    // lame refuses to accept somegenres that come from cddb, and users get upset
+    if (false && (genre != NULL) && (strlen(genre) > 0))
     {
         args[pos++] = "--tg";
         args[pos++] = genre;
@@ -399,7 +405,7 @@ void lame(int tracknum,
                 size = 1;
             }
             
-        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0));
+        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0) && (pos < 256));
         buf[pos] = '\0';
         
         if (sscanf(buf, "%d/%d", &sector, &end) == 2)
@@ -412,9 +418,7 @@ void lame(int tracknum,
     /* don't go on until the signal for the previous call is handled */
     while (lame_pid != 0)
     {
-#ifdef DEBUG
-        printf("w4");
-#endif
+        debugLog("w4\n");
         usleep(100000);
     }
     
@@ -508,7 +512,7 @@ void oggenc(int tracknum,
                 size = 1;
             }
             
-        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0));
+        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0) && (pos < 256));
         buf[pos] = '\0';
 
         if (sscanf(buf, "\t[\t%d.%d%%]", &sector, &end) == 2)
@@ -525,9 +529,7 @@ void oggenc(int tracknum,
     /* don't go on until the signal for the previous call is handled */
     while (oggenc_pid != 0)
     {
-#ifdef DEBUG
-        printf("w5");
-#endif
+        debugLog("w6\n");
         usleep(100000);
     }
     
@@ -675,7 +677,7 @@ void flac(int tracknum,
                 size = 1;
             }
             
-        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0));
+        } while ((buf[pos] != '\r') && (buf[pos] != '\n') && (size > 0) && (pos < 256));
         buf[pos] = '\0';
 
         for (; pos>0; pos--)
@@ -698,9 +700,92 @@ void flac(int tracknum,
     /* don't go on until the signal for the previous call is handled */
     while (flac_pid != 0)
     {
-#ifdef DEBUG
-        printf("w6");
-#endif
+        debugLog("w7\n");
         usleep(100000);
     }
 }
+
+void wavpack(int tracknum,
+             char * wavfilename,
+             int compression,
+             bool hybrid,
+             int bitrate,
+             double * progress)
+{
+    const char * args[10];
+    int fd;
+    int pos;
+    int size;
+    char buf[256];
+    
+    pos = 0;
+    args[pos++] = "wavpack";
+    
+    if(hybrid)
+    {
+        char bitrateTxt[7];
+        snprintf(bitrateTxt, 7, "-b%d", bitrate);
+        args[pos++] = bitrateTxt;
+        
+        args[pos++] = "-c";
+    }
+    
+    args[pos++] = "-y";
+    if(compression == 0)
+        args[pos++] = "-f";
+    else if(compression == 2)
+        args[pos++] = "-h";
+    else if(compression == 3)
+        args[pos++] = "-hh";
+    // default is no parameter (normal compression)
+    
+    args[pos++] = "-x3";
+    
+    args[pos++] = wavfilename;
+    args[pos++] = NULL;
+    
+    fd = exec_with_output(args, STDERR_FILENO, &wavpack_pid);
+    
+    do
+    {
+        pos = -1;
+        do
+        {
+            pos++;
+            size = read(fd, &buf[pos], 1);
+            
+            if (size == -1 && errno == EINTR)
+            /* signal interrupted read(), try again */
+            {
+                pos--;
+                size = 1;
+            }
+        } while ((buf[pos] != '\b') && (size > 0) && (pos < 256));
+        
+        buf[pos] = '\0';
+        
+        for (; pos>0; pos--)
+        {
+            if (buf[pos] == ',')
+            {
+                pos++;
+                break;
+            }
+        }
+        
+        int percent;
+        if (sscanf(&buf[pos], "%d%%", &percent) == 1)
+        {
+            *progress = (double)percent/100;
+        }
+    } while (size > 0);
+    
+    close(fd);
+    /* don't go on until the signal for the previous call is handled */
+    while (wavpack_pid != 0)
+    {
+        debugLog("w8\n");
+        usleep(100000);
+    }
+    
+}
